@@ -18,11 +18,11 @@ App::Prove - Implements the C<prove> command.
 
 =head1 VERSION
 
-Version 3.43_01
+Version 3.43_02
 
 =cut
 
-our $VERSION = '3.43_01';
+our $VERSION = '3.43_02';
 
 =head1 DESCRIPTION
 
@@ -344,13 +344,13 @@ sub _get_args {
     # Handle verbose, quiet, really_quiet flags
     my %verb_map = ( verbose => 1, quiet => -1, really_quiet => -2, );
 
-    my @verb_adj = grep {$_} map { $self->$_() ? $verb_map{$_} : 0 }
+    my @verb_adj = map { $self->$_() ? $verb_map{$_} : () }
       keys %verb_map;
 
     die "Only one of verbose, quiet or really_quiet should be specified\n"
       if @verb_adj > 1;
 
-    $args{verbosity} = shift @verb_adj || 0;
+    $args{verbosity} = shift @verb_adj if @verb_adj;
 
     for my $a (qw( merge failures comments timer directives normalize )) {
         $args{$a} = 1 if $self->$a();
